@@ -6,7 +6,6 @@
 # Revisions:    11/25/2003 - P. Harvey Created
 #               12/02/2003 - P. Harvey Completely reworked and figured out many
 #                            more tags
-#               01/19/2004 - P. Harvey Added CleanRaw()
 #
 # References:   1) http://www.cybercom.net/~dcoffin/dcraw/
 #               2) http://www.wonderland.org/crw/
@@ -22,7 +21,7 @@ use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 use Image::ExifTool::Canon;
 
-$VERSION = '1.49';
+$VERSION = '1.54';
 
 sub WriteCRW($$);
 sub ProcessCanonRaw($$$);
@@ -269,7 +268,6 @@ sub BuildMakerNotes($$$$$$);
         {
             # D30
             Name => 'SerialNumber',
-            Description => 'Camera Body No.',
             Condition => '$$self{Model} =~ /EOS D30\b/',
             Writable => 'int32u',
             PrintConv => 'sprintf("%x-%.5d",$val>>16,$val&0xffff)',
@@ -278,7 +276,6 @@ sub BuildMakerNotes($$$$$$);
         {
             # all EOS models (D30, 10D, 300D)
             Name => 'SerialNumber',
-            Description => 'Camera Body No.',
             Condition => '$$self{Model} =~ /EOS/',
             Writable => 'int32u',
             PrintConv => 'sprintf("%.10d",$val)',
@@ -314,10 +311,13 @@ sub BuildMakerNotes($$$$$$);
     0x1814 => { #3
         Name => 'MeasuredEV',
         Notes => q{
-            this the Canon name for what could better be called MeasuredLV, and is
-            offset by about -5 EV from the calculated LV for most models
+            this is the Canon name for what could better be called MeasuredLV, and
+            should be close to the calculated LightValue for a proper exposure with most
+            models
         },
         Format => 'float',
+        ValueConv => '$val + 5',
+        ValueConvInv => '$val - 5',
     },
     0x1817 => {
         Name => 'FileNumber',
@@ -328,6 +328,7 @@ sub BuildMakerNotes($$$$$$);
     },
     0x1818 => { #3
         Name => 'ExposureInfo',
+        Groups => { 1 => 'CIFF' }, # (only so CIFF shows up in group lists)
         Writable => 0,
         SubDirectory => {
             TagTable => 'Image::ExifTool::CanonRaw::ExposureInfo',
@@ -553,8 +554,7 @@ sub BuildMakerNotes($$$$$$);
         ValueConv => 'abs($val)<100 ? 1/(2**$val) : 0',
         ValueConvInv => '$val>0 ? -log($val)/log(2) : -100',
         PrintConv => 'Image::ExifTool::Exif::PrintExposureTime($val)',
-        # do eval to convert things like '1/100'
-        PrintConvInv => 'eval $val',
+        PrintConvInv => 'Image::ExifTool::Exif::ConvertFraction($val)',
     },
     2 => {
         Name => 'ApertureValue',
@@ -652,7 +652,7 @@ sub ProcessCanonRaw($$$)
     # read the directory (10 bytes per entry)
     $raf->Read($buff, 10 * $entries) == 10 * $entries or return 0;
 
-    $verbose and $exifTool->VerboseDir('Raw', $entries);
+    $verbose and $exifTool->VerboseDir('CIFF', $entries);
     my $index;
     for ($index=0; $index<$entries; ++$index) {
         my $pt = 10 * $index;
@@ -837,7 +837,7 @@ sub ProcessCRW($$)
     $buildMakerNotes and InitMakerNotes($exifTool);
 
     # set the FileType tag unless already done (ie. APP0 CIFF record in JPEG image)
-    $exifTool->SetFileType() unless $exifTool->GetValue('FileType');
+    $exifTool->SetFileType();
 
     # build directory information for main raw directory
     my %dirInfo = (
@@ -894,7 +894,7 @@ tags.)
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
